package dev.pfaff.altargui.improv;

import dev.pfaff.altargui.util.listenable.Listenable;
import dev.pfaff.altargui.util.listenable.ListenableValue;
import dev.pfaff.altargui.util.Sink;

import java.util.List;

public interface MultiChildSupplier {
	Listenable listenable();

	void forEach(Sink<Element> consumer);

	static MultiChildSupplier of(List<Element> elements) {
		record Impl(List<Element> elements) implements MultiChildSupplier {

			@Override
			public Listenable listenable() {
				return listener -> {
					listener.run();
					return () -> {};
				};
			}

			@Override
			public void forEach(Sink<Element> consumer) {
				consumer.begin(this.elements.size());
				this.elements.forEach(consumer);
				consumer.end();
			}
		}
		return new Impl(List.copyOf(elements));
	}

	static MultiChildSupplier of(ListenableValue<List<Element>> elements) {
		return new MultiChildSupplier() {
			@Override
			public Listenable listenable() {
				return elements;
			}

			@Override
			public void forEach(Sink<Element> consumer) {
				var list = elements.value();
				consumer.begin(list.size());
				list.forEach(consumer);
				consumer.end();
			}
		};
	}
}
